/*
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef MAIN_H
using namespace std; 
#define MAIN_H

#define DEFAULT_MANGLED_POSTFIX ".code"
#define DEFAULT_COMMENTS_POSTFIX ".comments"

/* DO NOT EDIT BELOW THIS LINE */
#define BUF_LENGTH 512
#define FILE_NAME_LENGTH 200
#define LAST_READ_LENGTH 3
#define LAST_WRITTEN_LENGTH 3
#define BYTES_PER_K 1024.0
#define FLAG_HISTORY_MAX 5
#define DEFINE_SEARCH_PRECISION 2

#include <fstream>
#include <iostream>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "func.h"
#include "io.h"
#include "dformat.h"
#endif
